/* Nilo - A Network boot loader.

   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the PXE types */
//#include "pxe_types.h"

/* Define the data structure dumping functions for debug use */
// #include "dump.h"

#include "nilo.h"

#include "tagged.h"
#include "util.h"

#ifdef HAVE_STDDEF_H
#include <stddef.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include <fcntl.h>

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#else
extern int optind;
extern char *optarg;
#endif

int verbosity;

extern void trace_msg(const char *fmt, ...);
extern void trace_printf(const char *fmt, ...);
struct client_data *client_info;
static char *infile;
#define MAX_TRACE_BUF_SIZE      128

void verbose (int level, const char *fmt, ...);
void dump_tagged (void);

static void
usage (const char *prog)
{
  fprintf (stderr, "This program dumps the tagged image header.\n");
  fprintf (stderr, "Usage: %s [hvi] tagged-file\n", prog);
  fprintf (stderr, "-h\tHelp\n");
  fprintf (stderr, "-v\tVerbose mode\n");
  fprintf (stderr, "-i\tTagged input file\n");
  exit (-1);
}

main (int argc, char **argv)
{
    int c;
    int infile_fd;

    while ((c = getopt (argc, argv, "phvi:")) != -1) {
        switch (c) {

        case 'i':
            infile = optarg;
//            verbose (1, "Infile is %s.", infile);
            break;

        case 'h':
            usage (argv[0]);
            break;

        case 'v':
            verbosity++;
            break;
 
        default:
            usage (argv[0]);
            break;
        }
    }
//      printf ("non-option ARGV-elements: ");
//      while (optind < argc)
//          printf ("%s ", argv[optind++]);
//      printf ("\n");
    if (optind < argc)
        infile = argv[optind++];
    verbose (1, "The input file is %s.\n", infile);

    /* FIXME: This is just the default size */
    client_info = (struct client_data *)malloc (sizeof(struct client_data));
    client_info->tftp_blksize = 512;

    dump_tagged();
}

/* This reads in a tagged image boot file, and decodes the header fields */
void
dump_tagged (void)
{
    int i, x, vlen, nlen, flags;
    char *vinfo;
    int infile_fd;
    union infoblock dirblk;
    struct imgheader *imghead;
    struct segheader *seghead, *foo;
    struct segheader sh = { 0, 0, 0, 0, 0L, 0L, 0L };

    infile_fd = open (infile, O_RDONLY);

    x = read (infile_fd, (char *)&dirblk, sizeof(union infoblock));
    
    if (dirblk.i.magic != TAG_MAGIC || dirblk.s[255] != 0xAA55) {
        verbose (0, "ERROR: Tagged file magic number is incorrect.");
        exit(0);
    } else {
        verbose (2, "This is a tagged bootfile.");
    }

    imghead = (struct imgheader *)&dirblk;
    verbose (1, "The Vendor length is 0x%02x bytes\n\tnon-vendor length is 0x%02x bytes."
             , imghead->vlen*4, imghead->nlen*4);
    vinfo = (char *)&dirblk + sizeof(struct imgheader);
    
    verbose (1, "Image %s return to NILO after execution.",
             (imghead->bootret) ? "can" : "can't");
    verbose (1, "Location: 0x%04hx:0x%04hx\n\tExecaddr: 0x%04hx:0x%04hx",
           imghead->location.offset, imghead->location.segment,
           imghead->execaddr.offset, imghead->execaddr.segment);
    if (imghead->vlen > 0)
        verbose (1, "Vendor info: %s", vinfo);

//    seghead = (struct segheader *)&dirblk + sizeof(struct imgheader) + (imghead->vlen * sizeof(int) - 1);
//    seghead = (struct segheader *)(dirblk.l+imghead->vlen + imghead->nlen);
    
    /* get the first segment */
    seghead = (struct segheader *)(((char *)&dirblk) + (imghead->vlen*4)
                                   + (imghead->nlen*4));
    for (i=1; i<=10; i++) {
        verbose (1, "\n\tSegment header #%d.", i);
        if (i>0)
            seghead = (struct segheader *)(((char *)&dirblk) + (seghead->vlen*4)
                                           + (seghead->nlen*4)
                                           + (i*(sizeof(struct segheader))));
        if (seghead->vlen)
            vinfo = (char *)(seghead + sizeof(struct segheader));

        verbose (1, "The Segment Header Vendor length is 0x%02x bytes.",
                 seghead->vlen*4);
        verbose (1, "The Segment Header non-vendor length is 0x%02x bytes.",
                 seghead->nlen*4);
        verbose (1, "The Segment Vendor Tag is 0x%02x.", seghead->vendorinfo);
        /* FIXME: These are likely to be Linux specific */
        switch (seghead->vendorinfo) {
        case VTAG_BOOT_LOADER:
            verbose (1, "\tLinux primary boot loader.");
            break;
        case VTAG_CMD_LINE:
            verbose (1, "\tLinux command line.");
            break;
        case VTAG_FLOPPY_SECTOR:
            verbose (1, "\tLinux floppy boot sector.");
            break;
        case VTAG_KERNEL_SETUP:
            verbose (1, "\tLinux kernel setup.");
            break;
        case VTAG_KERNEL_IMAGE:
            verbose (1, "\tLinux kernel image.");
            break;
        case VTAG_RAMDISK:
            verbose (1, "\tLinux ramdisk image.");
            break;
        }

        verbose (1, "The Segment flags are 0x%01x.", seghead->flags);
        switch (seghead->flags) {
        case LOAD_ABSOLUTE:
            verbose (1, "\tLoading to an absolute 32 bit address.");
            break;
        case LOAD_SUB_ADDR:
            verbose (1, "\tLoading to the last writeable location in memory.");
            break;
        case LOAD_ADD_ADDR:
            verbose (1, "\tLoad address is 1 past the last image.");
            break;
        case LOAD_SUB_START:
            verbose (1, "\tLoad address is subtracted from the start of the last image.");
            break;
        }
        verbose (1, "The Segment endmark %s set.", (seghead->endmark) ? "is" : "isn't");
        verbose (1, "The load address for this segment is 0x%08x.", seghead->loadaddr);
        verbose (1, "The image length for this segment is 0x%08x bytes.", seghead->imglength);
        verbose (1, "The memory length for this segment is 0x%08x bytes.", seghead->memlength);
        
         if (seghead->vlen > 0) {
            /* if we're a ramdisk image */
            if (seghead->vendorinfo == 0x15) {
                switch (*vinfo) {
                case RD_AUTO:
                    verbose (1, "\tVendor info: RD: auto positioning.");
                    break;
                case RD_EOM:
                    verbose (1, "\tVendor info: positioned by Boot-Rom.");
                    break;
                case RD_FIXED:
                    verbose (1, "\tVendor info: RD: fixed memory location.");
                    break;
                }
            }
        } else {
            verbose (1, "There is no vendor info.");
        }

        if (seghead->endmark)
            break;
    } /* for i */

    close (infile_fd);
}

void
store(u_int32_t *d, u_int8_t *s, int n)
{
    printf ("Storing %d bytes at %08x\r\n", n, d);
    
}

void
panic(const char *fmt, ...)
{
        va_list args;

        printf("PANIC: ");
        va_start(args, fmt);
        vprintf(fmt, args);
        va_end(args);
        printf("\n");
        exit (1);
}

const unsigned char hexchars[]="0123456789abcdef";

void
verbose (int level, const char *fmt, ...)
{
    va_list ap;
    char tmp[MAX_TRACE_BUF_SIZE];
    char buf[20];
    const char *errmsg;
    
    va_start (ap, fmt);
    vsprintf (tmp, fmt, ap);

    if (level <= verbosity)
        printf ("\t%s\r\n", tmp);
    fflush (stdout);
    va_end (ap);
}

/* Convert a buffer into an ascified representation,
   ie.. ^C is 0x03, etc... */
unsigned char *
ascify_buffer (unsigned char *p, const unsigned char *s, int x)
{
    int i;
    unsigned char *p1 = p;

    /* convert some characters so it'll look right when
       we display it */
    for (i=0 ; i<x; i++) {
        switch (s[i]) {
        case '\012':                    /* newlines */
            *p++ = '\\';
            *p++ = 'n';
            continue;
        case '\015':                    /* carriage returns */
            *p++ = '\\';
            *p++ = 'r';
            continue;
        case '\033':                    /* escape */
            *p++ = '\\';
            *p++ = 'e';
            continue;
        case '\011':                    /* tab */
            *p++ = '\\';
            *p++ = 't';
            continue;
        case '\010':                    /* backspace */
            *p++ = '\\';
            *p++ = 'b';
            continue;
        }
        
        if (s[i] < 26) {                /* modify control characters */
            *p++ = '^';
            *p++ = s[i] + '@';
            continue;
        }
        if (s[i] >= 128) {              /* modify control characters */
            *p++ = '!';
            *p++ = s[i] + '@';
            continue;
        }
        if (s[i] > 26 && s[i] < 128) {  /* no change */
            *p++ = s[i];
            continue;
        }
    }

    *p = '\0';                          /* terminate the string */

    return p1;
}

/* Convert each byte into it's hex represntation */
unsigned char *
hexify_buffer (const unsigned char *s, int x)
{
    int i;
    unsigned char *p;
    static unsigned char string[MAX_TRACE_BUF_SIZE];

    p = string;

    /* convert some characters so it'll look right when displayed */
    for (i=0 ; i<x; i++) {
        /* use the hex value */
      if  (s[i] < 1) {
	*p++ = '*';
        *p++ = ' ';
      } else {
        *p++ = hexchars[s[i] >> 4];
        *p++ = hexchars[s[i] & 0xf];
        *p++ = ' ';
      }
    }

    *p = '\0';                                  // terminate the string
    return string;
}

/* convert number NIB to a hex digit.
      param is a decimal digit.
      returns a hex digit. */
char
digit2hex(int digit)
{
  if (digit < 10)
    return '0' + digit;
  else
    return 'a' + digit - 10;
}

/* convert an ascii hex digit to a number.
      param is hex digit.
      returns a decimal digit. */
int
hex2digit (int digit)
{
if (digit == 0)
  return 0;

  if (digit >= '0' && digit <= '9')
    return digit - '0';
  if (digit >= 'a' && digit <= 'f')
    return digit - 'a' + 10;
  if (digit >= 'A' && digit <= 'F')
    return digit - 'A' + 10;

  /* shouldn't ever get this far */
  return -1;
}

/* These are just stubs for any object files we link in */
void
trace_msg (const char *fmt, ...)
{
    va_list ap;
    char tmp[MAX_TRACE_BUF_SIZE];
    char buf[20];
    const char *errmsg;
    
    va_start (ap, fmt);
    vsprintf (tmp, fmt, ap);

    printf ("%s\n", tmp);

    fflush (stdout);
    va_end (ap);
}

void
trace_printf (const char *fmt, ...)
{
    va_list ap;
    char tmp[MAX_TRACE_BUF_SIZE];
    char buf[20];
    const char *errmsg;
    
    va_start (ap, fmt);
    vsprintf (tmp, fmt, ap);

    printf ("%s", tmp);

    fflush (stdout);
    va_end (ap);
}


