/* 
   Copyright (C) 1999 Free Software Foundation, Inc.

This file is part of Nilo.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef __PXE_UDP_H__
#define __PXE_UDP_H__

/* This is generated by running configure */
#include "niloconfig.h"

/* Define the basic data types */
#include <sys/types.h>

/* Define the PXE data types */
#include "pxe_types.h"

/* Constants that define the offsets into the table of functions.
 * One of these must be loaded into the BX register before making
 * a call a UDP API service. 
 */
#define	PXENV_UDP_OPEN	0x30
#define	PXENV_UDP_CLOSE	0x31
#define	PXENV_UDP_READ	0x32
#define	PXENV_UDP_WRITE	0x33

/* Data structure for pxenv_udp_open().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 *     src_ip - The IP number of this node.
 */
typedef struct pxenv_udp_open_s {
    pxenv_status_t status;
    ip4_t src_ip;
} pxenv_udp_open_t;

/* Data structure for pxenv_udp_close().
 * parameters:
 *     none
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_udp_close_s {
    pxenv_status_t status;
} pxenv_udp_close_t;


/* Data structure for pxenv_udp_read().
 * parameters:
 *     dest_ip - if non-zero, filter all incoming packets and
 *               receive the ones for this IP address. If
 *               UDP_READ_ANY_IP (zero), then read any packet.
 *     d_port - if non-zero, filter all incoming packets and
 *               receive the ones for this UDP port. If
 *               UDP_READ_ANY_PORT (zero), then read any packet.
 *     buffer_size - size of the receive buffer.
 *     buffer_off - segment offset.
 *     buffer_seg - the segment the buffer is in.
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 *     src_ip - The IP number of the sending node.
 *     dest_ip - If this parameter was UDP_READ_ANY_IP (zero), then
 *              return the destination IP number.
 *     s_port - The UDP port number of the sending node.
 *     d_port - If this parameter was zero, then return the destination
 *              UDP port.
 *     buffer_size - Number of bytes written into the receive buffer.
 */
typedef struct pxenv_udp_read_s {
    pxenv_status_t status;
    ip4_t src_ip;
    ip4_t dest_ip;
    udp_port_t s_port;
    udp_port_t d_port;
    u_int16_t buffer_size;
    addr32_t *buffer;
} pxenv_udp_read_t;

/* Accept packets sent to any IP number. */
#define	UDP_READ_ANY_IP	0x0000

/* Accept packets sent to any UDP port. */
#define	UDP_READ_ANY_PORT 0x0000

/* Accept packets only when sent to a specific IP address. */
#define	UDP_READ_CHECK_IP 0x0001


/* Data structure for pxenv_udp_write().
 * parameters:
 *     dest_ip - the IP number of the destination.
 *     gw_ip - the ip number of the gateway.
 *     src_port - the UDP port of the sendor, defaults to 2069 if this
 *              flag is zero.
 *     dest_port - the UDP port of the destination.
 *     buffer_size - size of the transmit buffer.
 *     buffer - the buffer address in offset:segment format.
 * returns:
 *     status - The exit status of this function. This is set to one
 *	        of the PXENV_STATUS_xxx constants.
 */
typedef struct pxenv_udp_write_s {
    pxenv_status_t status;
    ip4_t dest_ip;
    ip4_t gw_ip;
    udp_port_t src_port;
    udp_port_t dest_port;
    u_int16_t buffer_size;
    addr32_t *buffer;
} pxenv_udp_write_t;

/* Prototypes for the UDP functions */

pxenv_ret_code_t
pxenv_udp_open (pxenv_udp_open_t *udpopen);
pxenv_ret_code_t
pxenv_udp_close (pxenv_udp_close_t *udpclose);
pxenv_ret_code_t
pxenv_udp_read (pxenv_udp_read_t *udpread);
pxenv_ret_code_t
pxenv_udp_write (pxenv_udp_write_t *udpwrite);

#endif /* __PXE_UDP_H__ */

